@@ -1,5 +1,6 @@
 #!/usr/bin/env bash
-# lesspipe.sh, a preprocessor for less (version 2.06)
+# lesspipe.sh, a preprocessor for less
+lesspipe_version=2.07
 # Author: Wolfgang Friebel (wp.friebel AT gmail.com)
 #( [[ -n 1 && -n 2 ]] ) > /dev/null 2>&1 || exec zsh -y --ksh-arrays -- "$0" ${1+"$@"}
 
@@ -75,6 +76,13 @@ filetype () {
 			[[ $ftype == mpeg ]] && ftype=mp3 ;;
 	esac
 	### get file type from 'file' command for an unspecific result
+	if [[ "$fcat" == message && $ftype == plain ]]; then
+		ftype=msg
+	fi
+	if [[ "$fcat" == message && $ftype == rfc822 ]]; then
+		fcat=text
+		ftype=email
+	fi
 	if [[ "$fcat" == application && "$ftype" == octet-stream || "$fcat" == text && $ftype == plain ]]; then
 		ft=$(file -L -s -b "$1" 2> /dev/null)
 		# first check if the file command yields something
@@ -95,8 +103,10 @@ filetype () {
 				ftype=pgp ;;
 			Audio\ file\ with\ ID3\ *)
 				ftype=mp3 ;;
-			'OpenOffice.org 1.x Writer document')
+			OpenOffice.org\ 1.x\ Writer\ document)
 				ftype=ooffice1 ;;
+			*osascript*)
+				ftype=applescript ;;
 			# if still unspecific, determine file type by extension
 			data)
 				### binary only file formats, type not guessed by 'file'
@@ -141,12 +151,18 @@ filetype () {
 			esac
 		fi
 	fi
+
 	echo "$ftype:$fchar:$fcat"
 }
 
 msg () {
 	[[ -n "$LESSQUIET" ]] && return;
-	echo "==> $*"
+	if [[ -n "$lesspipe_version" ]]; then
+		echo "==> (lesspipe $lesspipe_version) $*"
+	else
+		echo "==> $*"
+	fi
+	lesspipe_version=
 }
 
 contentline () {
@@ -310,7 +326,7 @@ get_unpack_cmd () {
 		trans=()
 		echo ""|iconv --byte-subst - 2>/dev/null && rep=(--unicode-subst="$qm" --byte-subst="$qm" --widechar-subst="$qm") # MacOS
 		echo ""|iconv -f "$fchar" -t "$locale//TRANSLIT" - 2>/dev/null && trans=(-t "$locale//TRANSLIT")
-		msg "append $sep$sep to filename to view the $fchar encoded file"
+		msg "append $sep$sep to filename to view the original $fchar encoded file"
 		cmd=(iconv "${rep[@]}" -f "$fchar" "${trans[@]}" "$2")
 		# loop protection, just in case
 		lclocale=
@@ -379,11 +395,14 @@ analyze_args () {
 	# return if we want to watch growing files
 	[[ $lessarg == *less\ *\ +F\ * || $lessarg == *less\ *\ : ]] && exit 0
 	# color is set when calling less with -r or -R or LESS contains that option
-	lessarg="l $LESS $lessarg"
-	# shellcheck disable=SC2001
-	lessarg=$(echo "$lessarg" | sed 's/ -[a-zA-Z]*[rR]/ -r/')
+	typeset -l has_r
+	has_r="l $LESS $lessarg"
+	has_r=${has_r/[a-z]-/}
+	has_r=${has_r/--raw-control-chars/ -r}
+
+	has_r=${has_r//[a-qs-z]/}
 	has_cmd tput && colors=$(tput colors) || colors=0
-	if [[ $colors -ge 8 && $lessarg == *\ -[rR]* ]]; then
+	if [[ $colors -ge 8 && $has_r == *\ -r* ]]; then
 		COLOR="--color=always"
 	else
 		COLOR="--color=auto"
@@ -393,8 +412,6 @@ analyze_args () {
 }
 
 has_colorizer () {
-	arg="$1"
-	[[ "$arg" = - ]] && arg=
 	[[ $COLOR == *always ]] || return
 	[[ $2 == plain || -z $2 ]] && return
 	prog=${LESSCOLORIZER%% *}
@@ -407,18 +424,33 @@ has_colorizer () {
 		bat|batcat)
 			# only allow an explicitly requested language
 			[[ -z $3 ]] && opt=() || opt=(-l "$3")
-			opt+=("$COLOR" --style=plain --paging=never) ;;
+			[[ -n $LESSCOLORIZER && $LESSCOLORIZER = *\ *--style=* ]] && style="${LESSCOLORIZER/* --style=/}"
+			[[ -z $style ]] && style=$BAT_STYLE
+			[[ -z $style ]] && style=plain
+			[[ -n $LESSCOLORIZER && $LESSCOLORIZER = *\ *--theme=* ]] && theme="${LESSCOLORIZER/* --theme=/}"
+			[[ -z $theme ]] && theme=$BAT_THEME
+			[[ -z $theme ]] && theme=ansi
+			if [[ -r "$HOME/.config/bat/config" ]]; then
+				grep -q -e '^--style' "$HOME/.config/bat/config" || opt+=(--style="${style%% *}")
+				grep -q -e '^--theme' "$HOME/.config/bat/config" || opt+=(--theme="${theme%% *}")
+			fi
+			opt+=("$COLOR" --paging=never "$1") ;;
 		pygmentize)
 			pygmentize -l "$2" /dev/null &>/dev/null && opt=(-l "$2") || opt=(-g)
-			[[ -n $LESSCOLORIZER && $LESSCOLORIZER =~ pygmentize\ \ *-O\ *style=[a-z]* ]] && opt+=(-O "${LESSCOLORIZER##* }")
-			[[ $colors -ge 256 ]] && opt+=(-f terminal256) ;;
+			[[ -n $LESSCOLORIZER && $LESSCOLORIZER = *-O\ *style=* ]] && style="${LESSCOLORIZER/*style=/}"
+			[[ -n $style ]] && opt+=(-O style="${style%% *}")
+			[[ $colors -ge 256 ]] && opt+=(-f terminal256)
+			[[ "$1" == - ]] || opt+=("$1") ;;
 		source-highlight)
-			[[ -z $arg ]] && arg=/dev/stdin
+			arg="$1"
+			[[ -z $1 || "$1" == - ]] && arg="/dev/stdin"
 			[[ -n "${opt[*]}" && -n "$2" ]] && opt=(-s "$2") || opt=()
-			opt+=(--failsafe -f esc -i) ;;
+			style=esc
+			[[ $colors -ge 256 ]] && style=esc256
+			opt+=(--failsafe -f "$style" -i "$arg") ;;
 		code2color|vimcolor)
-			opt=()
-			[[ -n "$fileext" ]] && opt=(-l "$fileext") ;;
+			opt=("$1")
+			[[ -n "$fileext" ]] && opt=(-l "$fileext" "$1") ;;
 		*)
 			return ;;
 	esac
@@ -513,7 +545,7 @@ isfinal () {
 		sharedlib)
 			cmd=(istemp nm "$1");;
 		pod)
-			[[ -z $file2 ]] && LESSQUIET=1 &&
+			[[ -z $file2 ]] &&
 			{ { has_cmd pod2text && cmd=(pod2text "$1"); } ||
 			{ has_cmd perldoc && cmd=(istemp perldoc "$1"); }; } ;;
 		hdf|hdf5)
@@ -530,7 +562,8 @@ isfinal () {
 		pgp)
 			has_cmd gpg && cmd=(gpg -d "$1") ;;
 		plist)
-			has_cmd plistutil && cmd=(istemp "plistutil -i" "$1") ;;
+			{ has_cmd plistutil && cmd=(istemp "plistutil -i" "$1"); } ||
+			{ has_cmd plutil && cmd=(istemp "plutil -p" "$1"); } ;;
 		mp3)
 			has_cmd id3v2 && cmd=(istemp "id3v2 --list" "$1") ;;
 		log)
@@ -539,6 +572,9 @@ isfinal () {
 		csv)
 			{ has_cmd csvlook && cmd=(csvlook "$1"); } ||
 			{ has_cmd pandoc && cmd=(pandoc -f csv -t plain "$1"); } ;;
+		json)
+			[[ $COLOR = *always ]] && opt=(-C .) || opt=(.)
+			has_cmd jq && cmd=(jq "${opt[@]}" "$1") ;;
 	esac
 	fi
 	# not a specific file format
@@ -553,7 +589,7 @@ isfinal () {
 		fi
 	fi
 	if [[ -n ${cmd[*]} && "${cmd[*]}" != "cat" ]]; then
-		[[ -z $msg ]] && msg="append $sep to filename to view the $x file"
+		[[ -z $msg ]] && msg="append $sep to filename to view the original $x file"
 		msg "$msg"
 	fi
 	if [[ -n ${cmd[*]} ]]; then
@@ -564,11 +600,11 @@ isfinal () {
 		fi
 	else
 		[[ -n "$file2" ]] && fext="$file2"
-		[[ -z "$fext" && $fcat == text && $x != plain ]] && fext=$x
+		[[ $fcat == text && $x != plain ]] && fext=$x
 		[[ -z "$fext" ]] && fext=$(fileext "$fileext")
 		fext=${fext##*/}
 		[[ -z ${colorizer[*]} ]] && has_colorizer "$1" "$fext" "$file2"
-		[[ -n ${colorizer[*]} && $fcat != binary ]] && "${colorizer[@]}" "$1" && return
+		[[ -n ${colorizer[*]} && $fcat != binary ]] && "${colorizer[@]}" && return
 		# if fileext set, we need to filter to get rid of .fileext
 		[[ -n $fileext || "$1" == - || "$1" == "$t" ]] && cat "$1"
 	fi
@@ -613,12 +649,12 @@ isarchive () {
 				istemp "isoinfo -d -i" "$2"
 				isoinfo -d -i "$t"| grep -E '^Joliet' && joliet=J
 				contentline
-				isoinfo -fR$joliet -i "$t" ;;
+				isoinfo -fR"$joliet" -i "$t" ;;
 			7za|7zr)
 				istemp "$prog l" "$2"
 		esac
 	fi
-	[[ $? && $3 ]] && msg "could not retrieve $3 from $2"
+	[[ $? != 0 && -n $3 ]] && msg ":$!: could not retrieve $3 from $2"
 }
 
 cabextract2 () {
@@ -725,7 +761,7 @@ setopt sh_word_split 2>/dev/null
 PATH=$PATH:${0%%/lesspipe.sh}
 # the current locale in lowercase (or generic utf-8)
 locale=$(locale|grep LC_CTYPE|sed 's/.*"\(.*\)"/\1/') || locale=en_US.UTF-8
-lclocale=$(echo ${locale##*.}|tr '[:upper:]' '[:lower:]')
+lclocale=$(echo "${locale##*.}"|tr '[:upper:]' '[:lower:]')
 
 sep=:					# file name separator
 altsep='='				# alternate separator character
